/**
 * Copyright (c) 2019, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import React, { useEffect, useState } from "react";
import { Form } from "semantic-ui-react";
import { Field, GroupFields, InnerField, InnerGroupFields } from "./components";
import { isCheckBoxField, isDropdownField, isInputField, isRadioField, isTextField } from "./helpers/typeguards";
/**
 * This is a Forms component
 */
export const Forms = (props) => {
    const { onSubmit, triggerReset, children } = props;
    const [form, setForm] = useState(new Map());
    const [validFields, setValidFields] = useState(new Map());
    const [requiredFields, setRequiredFields] = useState(new Map());
    const [isSubmitting, setIsSubmitting] = useState(false);
    const formFields = [];
    /**
     * Handler for the onChange event
     * @param value
     * @param name
     */
    const handleChange = (value, name) => {
        const tempForm = new Map(form);
        tempForm.set(name, value);
        setForm(tempForm);
    };
    /**
     * Handler for the onChange event of checkboxes
     * @param value
     * @param name
     */
    const handleChangeCheckBox = (value, name) => {
        const tempForm = new Map(form);
        const selectedItems = tempForm.get(name);
        let itemIndex = -1;
        selectedItems.forEach((item, index) => {
            if (item === value) {
                itemIndex = index;
            }
        });
        itemIndex === -1 ? selectedItems.push(value) : selectedItems.splice(itemIndex, 1);
        tempForm.set(name, selectedItems);
        setForm(tempForm);
    };
    /**
     * Handler for the onBlur event
     * @param event
     * @param name
     */
    const handleBlur = (event, name) => {
        const tempRequiredFields = new Map(requiredFields);
        const tempValidFields = new Map(validFields);
        validate(name, tempRequiredFields, tempValidFields);
        setValidFields(tempValidFields);
        setRequiredFields(tempRequiredFields);
    };
    /**
     * Checks if the field has any errors (required but not filled | not validated)
     * @param inputField
     */
    const checkError = (inputField) => {
        if (isInputField(inputField)
            && !isRadioField(inputField)
            && inputField.required
            && !requiredFields.get(inputField.name)
            && isSubmitting) {
            return {
                errorMessages: [inputField.requiredErrorMessage],
                isError: true
            };
        }
        else if (isTextField(inputField) &&
            validFields.get(inputField.name) &&
            !validFields.get(inputField.name).isValid &&
            isSubmitting) {
            return {
                errorMessages: validFields.get(inputField.name).errorMessages,
                isError: true
            };
        }
        else {
            return {
                errorMessages: [],
                isError: false
            };
        }
    };
    /**
     * Resets form
     */
    const reset = () => {
        setIsSubmitting(false);
        init(true);
    };
    /**
     * Handles reset button click
     * @param event
     */
    const handleReset = (event) => {
        event.preventDefault();
        reset();
    };
    /**
     * Parses the children and
     * 1.passes form event handler functions as props to all the Field components
     * 2.extracts the props of the Field components
     * @param elements
     * @param fields
     */
    const parseChildren = (elements, fields) => {
        return React.Children.map(elements, (element) => {
            if (element.type === Field) {
                fields.push(element.props);
                return React.createElement(InnerField, {
                    formProps: {
                        checkError,
                        form,
                        handleBlur,
                        handleChange,
                        handleChangeCheckBox,
                        handleReset
                    },
                    passedProps: Object.assign({}, element.props)
                });
            }
            else if (element.type === GroupFields) {
                return React.createElement(InnerGroupFields, Object.assign(Object.assign({}, element.props), { children: parseChildren(element.props.children, fields) }));
            }
            else if (element.props
                && element.props.children
                && React.Children.count(element.props.children) > 0) {
                return React.createElement(element.type, Object.assign(Object.assign({}, element.props), { children: parseChildren(element.props.children, fields) }));
            }
            else {
                return element;
            }
        });
    };
    const mutatedChildren = [...parseChildren(children, formFields)];
    /**
     * Initializes the state of the from every time the passed formFields prop changes
     */
    useEffect(() => {
        init(false);
    }, [children]);
    /**
     * passes the reset function as an argument into the triggerReset prop method
     */
    useEffect(() => {
        if (triggerReset) {
            triggerReset(reset);
        }
        return () => {
            if (triggerReset) {
                triggerReset(() => undefined);
            }
        };
    }, []);
    /**
     * Initialize form
     * @param {boolean} isReset
     */
    const init = (isReset) => {
        const tempForm = new Map(form);
        const tempRequiredFields = new Map(requiredFields);
        const tempValidFields = new Map(validFields);
        formFields.forEach((inputField) => {
            if (isInputField(inputField)) {
                if (!tempForm.has(inputField.name) || isReset) {
                    inputField.value && !isReset
                        ? tempForm.set(inputField.name, inputField.value)
                        : (isRadioField(inputField) || isDropdownField(inputField)) && inputField.default
                            ? tempForm.set(inputField.name, inputField.default)
                            : isCheckBoxField(inputField)
                                ? tempForm.set(inputField.name, [])
                                : tempForm.set(inputField.name, "");
                }
                ((!inputField.value && (!tempForm.get(inputField.name) || !(tempForm.get(inputField.name).length > 0)))
                    || isReset) && inputField.required
                    ? tempRequiredFields.set(inputField.name, false)
                    : tempRequiredFields.set(inputField.name, true);
                tempValidFields.set(name, { isValid: true, errorMessages: [] });
            }
        });
        if (!isReset) {
            setRequiredFields(tempRequiredFields);
        }
        setForm(tempForm);
        setValidFields(tempValidFields);
    };
    /**
     * This function checks if a form field is valid
     * @param name
     * @param requiredFieldsParam
     * @param validFieldsParam
     */
    const validate = (name, requiredFieldsParam, validFieldsParam) => {
        const inputField = formFields.find((formField) => {
            return isInputField(formField) && formField.name === name;
        });
        if (!isCheckBoxField(inputField)) {
            form.get(name) !== null && form.get(name) !== ""
                ? requiredFieldsParam.set(name, true)
                : requiredFieldsParam.set(name, false);
        }
        else {
            form.get(name) !== null && form.get(name).length > 0
                ? requiredFieldsParam.set(name, true)
                : requiredFieldsParam.set(name, false);
        }
        const validation = {
            errorMessages: [],
            isValid: true
        };
        if (isTextField(inputField)
            && inputField.validation) {
            inputField.validation(form.get(name), validation, new Map(form));
        }
        validFieldsParam.set(name, {
            errorMessages: validation.errorMessages,
            isValid: validation.isValid
        });
    };
    /**
     * Handler for onSubmit event
     * @param event
     */
    const handleSubmit = (event) => {
        event.preventDefault();
        if (checkRequiredFieldsFilled() && checkValidated()) {
            setIsSubmitting(false);
            onSubmit(form);
        }
        else {
            setIsSubmitting(true);
        }
    };
    /**
     * Checks if all the required fields are filled
     */
    const checkRequiredFieldsFilled = () => {
        let requiredFilled = true;
        requiredFields.forEach((requiredFieldParam) => {
            if (!requiredFieldParam) {
                requiredFilled = false;
            }
        });
        return requiredFilled;
    };
    /**
     * Checks if all the fields are validated
     */
    const checkValidated = () => {
        let isValidated = true;
        validFields.forEach((validField) => {
            if (!validField.isValid) {
                isValidated = false;
            }
        });
        return isValidated;
    };
    return React.createElement(Form, { onSubmit: handleSubmit }, mutatedChildren);
};
//# sourceMappingURL=forms.js.map